#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file builds the ecTrans dwarf.
# ------------------------------------------------------------------------------

# Global color definitions.
NC='\033[0m'
RED='\033[1;31m'
GREEN='\033[1;32m'
YELLOW='\033[1;33m'

# Display fatal styled message with given argument.
function fatal {
    echo -e "\n$RED \u274c FATAL: $1$NC\n" 1>&2
    exit 1
}

# Display success styled message with given argument.
function success {
    echo -e "\n$GREEN \u2714 ~ $1$NC\n" 1>&2
}

# Display info styled message with given argument.
function info {
    echo -e "\n$YELLOW \u003F ~ $1$NC\n" 1>&2
}

# --

# Setup required modules.
source load_modules.sh

# Set compilers for make/cmake.
export FC90=ftn
export FC=ftn
export CC=cc
export CXX=CC

# Export used environment variables.
# Note that these are also used by the ecBuild repository.
export BASEDIR=$PWD
export SOURCEDIR=${BASEDIR}/sources
export BUILDDIR=${BASEDIR}/build
export INSTALLDIR=${BASEDIR}/install
export PATH=${PATH}:${INSTALLDIR}/ecbuild/bin/

# Create directories for the build process.
mkdir -p "${BASEDIR}" "${SOURCEDIR}" "${BUILDDIR}" "${INSTALLDIR}"

# Clonse and Install ecBuild.
info "\n\n==> INSTALLING ECBUILD.."
cd ${SOURCEDIR}
git clone https://github.com/ecmwf/ecbuild.git
cd ecbuild
git checkout "a1b6fe0"
# Remove obsolete switch '-Gfast'.
sed -i -e "s/-Gfast//" cmake/compiler_flags/Cray_Fortran.cmake
mkdir -p ${BUILDDIR}/ecbuild
cd ${BUILDDIR}/ecbuild
info "==>\t ECBUILD.."
${SOURCEDIR}/ecbuild/bin/ecbuild --prefix=${INSTALLDIR}/ecbuild ${SOURCEDIR}/ecbuild
info "==>\t MAKE.."
make
info "==>\t MAKE INSTALL.."
make install
success "==> SUCCESFULLY INSTALLED ECBUILD"

# Clone and Install FIAT.
info "\n\n==> INSTALLING FIAT.."
FIAT_DIR=fiat_cce16_rocm543
cd ${SOURCEDIR}
git clone https://github.com/ecmwf-ifs/fiat
cd fiat
git checkout "c4af45a"
# small fix to include OpenMP in linking of C programs
sed -i -e "s/target_link_libraries( fiat-printbinding OpenMP::OpenMP_C )/target_link_libraries( fiat-printbinding \$\{OpenMP_C_FLAGS\} OpenMP::OpenMP_C )/" src/programs/CMakeLists.txt
rm -rf ${BUILDDIR}/${FIAT_DIR}
mkdir -p ${BUILDDIR}/${FIAT_DIR}
cd ${BUILDDIR}/${FIAT_DIR}
info "==>\t ECBUILD.."
ecbuild -DCMAKE_BUILD_TYPE=RELEASE -DCMAKE_INSTALL_PREFIX=${INSTALLDIR}/${FIAT_DIR} -DENABLE_MPI=ON -DBUILD_SHARED_LIBS=OFF -DENABLE_TESTS=OFF ${SOURCEDIR}/fiat
info "==>\t MAKE.."
make -j16
info "==>\t MAKE INSTALL.."
make install
success "==> SUCCESFULLY INSTALLED FIAT"

# Clone and Install ecTrans
info "\n\n==> Installing ecTrans.."
ECTRANS_DIR=ectrans_cce16_rocm543
cd ${SOURCEDIR}
git clone https://github.com/ddegrauwe/ectrans.git
cd ectrans
git checkout "53848f4"
rm -rf ${BUILDDIR}/${ECTRANS_DIR} ${INSTALLDIR}/${ECTRANS_DIR}
mkdir -p ${BUILDDIR}/${ECTRANS_DIR}
cd ${BUILDDIR}/${ECTRANS_DIR}
info "==>\t ECBUILD.."
ecbuild --prefix=${INSTALLDIR}/${ECTRANS_DIR} -DCMAKE_BUILD_TYPE=RelWithDebInfo -Dfiat_ROOT=${INSTALLDIR}/${FIAT_DIR} -DBUILD_SHARED_LIBS=OFF -DENABLE_FFTW=OFF -DENABLE_GPU=ON -DENABLE_OMPGPU=OFF -DENABLE_ACCGPU=ON -DENABLE_TESTS=OFF -DENABLE_GPU_AWARE_MPI=ON -DENABLE_CPU=ON -DENABLE_ETRANS=ON  -DENABLE_DOUBLE_PRECISION=ON -DENABLE_SINGLE_PRECISION=OFF ${SOURCEDIR}/ectrans
info "==>\t MAKE.."
make -j32
info "==>\t MAKE (again).."
make -j32  # run twice because it fails after first time. Some wrong dependencies or something
info "==>\t MAKE INSTALL.."
make install
success "==> SUCCESFULLY INSTALLED ECTRANS"
